@@ -31,15 +31,15 @@
 
 #include "nautilus-program-choosing.h"
 #include "nautilus-link.h"
+#include "nautilus-window-slot-info.h"
+#include "nautilus-window-info.h"
+#include "nautilus-view.h"
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-string.h>
 #include <eel/eel-vfs-extensions.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkmenu.h>
-#include <gtk/gtkseparatormenuitem.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-uidefs.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <stdio.h>
 #include <string.h>
@@ -120,6 +120,76 @@ nautilus_drag_destroy_selection_list (GList *list)
 	g_list_free (list);
 }
 
+char **
+nautilus_drag_uri_array_from_selection_list (const GList *selection_list)
+{
+	GList *uri_list;
+	char **uris;
+
+	uri_list = nautilus_drag_uri_list_from_selection_list (selection_list);
+	uris = nautilus_drag_uri_array_from_list (uri_list);
+	eel_g_list_free_deep (uri_list);
+
+	return uris;
+}
+
+GList *
+nautilus_drag_uri_list_from_selection_list (const GList *selection_list)
+{
+	NautilusDragSelectionItem *selection_item;
+	GList *uri_list;
+	const GList *l;
+
+	uri_list = NULL;
+	for (l = selection_list; l != NULL; l = l->next) {
+		selection_item = (NautilusDragSelectionItem *) l->data;
+		if (selection_item->uri != NULL) {
+			uri_list = g_list_prepend (uri_list, g_strdup (selection_item->uri));
+		}
+	}
+
+	return g_list_reverse (uri_list);
+}
+
+char **
+nautilus_drag_uri_array_from_list (const GList *uri_list)
+{
+	const GList *l;
+	char **uris;
+	int i;
+
+	if (uri_list == NULL) {
+		return NULL;
+	}
+
+	uris = g_new0 (char *, g_list_length ((GList *) uri_list));
+	for (i = 0, l = uri_list; l != NULL; l = l->next) {
+		uris[i++] = g_strdup ((char *) l->data);
+	}
+	uris[i] = NULL;
+
+	return uris;
+}
+
+GList *
+nautilus_drag_uri_list_from_array (const char **uris)
+{
+	GList *uri_list;
+	int i;
+
+	if (uris == NULL) {
+		return NULL;
+	}
+
+	uri_list = NULL;
+
+	for (i = 0; uris[i] != NULL; i++) {
+		uri_list = g_list_prepend (uri_list, g_strdup (uris[i]));
+	}
+
+	return g_list_reverse (uri_list);
+}
+
 GList *
 nautilus_drag_build_selection_list (GtkSelectionData *data)
 {
@@ -228,6 +298,8 @@ nautilus_drag_file_local_internal (const char *target_uri_string,
 		g_object_unref (parent);
 	}
 	
+	g_object_unref (target);
+	
 	return result;
 }	
 
@@ -315,47 +387,45 @@ nautilus_drag_default_drop_action_for_netscape_url (GdkDragContext *context)
 }
 
 static gboolean
-check_same_fs (GFile *file1, GFile *file2)
+check_same_fs (NautilusFile *file1,
+	       NautilusFile *file2)
 {
-	GFileInfo *info1, *info2;
-	const char *id1, *id2;
-	gboolean res;
-	
-	info1 = g_file_query_info (file1,
-				   G_FILE_ATTRIBUTE_ID_FILESYSTEM,
-				   0, NULL, NULL);
+	char *id1, *id2;
+	gboolean result;
 
-	if (info1 == NULL) {
-		return FALSE;
+	result = FALSE;
+
+	if (file1 != NULL && file2 != NULL) {
+		id1 = nautilus_file_get_filesystem_id (file1);
+		id2 = nautilus_file_get_filesystem_id (file2);
+
+		if (id1 != NULL && id2 != NULL) {
+			result = (strcmp (id1, id2) == 0);
+		}
+
+		g_free (id1);
+		g_free (id2);
 	}
 
-	id1 = g_file_info_get_attribute_string (info1, G_FILE_ATTRIBUTE_ID_FILESYSTEM);
-	if (id1 == NULL) {
-		g_object_unref (info1);
+	return result;
+}
+
+static gboolean
+source_is_deletable (GFile *file)
+{
+	NautilusFile *naut_file;
+	gboolean ret;
+
+	/* if there's no a cached NautilusFile, it returns NULL */
+	naut_file = nautilus_file_get_existing (file);
+	if (naut_file == NULL) {
 		return FALSE;
 	}
 	
-	info2 = g_file_query_info (file2,
-				   G_FILE_ATTRIBUTE_ID_FILESYSTEM,
-				   0, NULL, NULL);
-	if (info2 == NULL) {
-		g_object_unref (info1);
-		return FALSE;
-	}
+	ret = nautilus_file_can_delete (naut_file);
+	nautilus_file_unref (naut_file);
 
-	id2 = g_file_info_get_attribute_string (info2, G_FILE_ATTRIBUTE_ID_FILESYSTEM);
-	if (id2 == NULL) {
-		g_object_unref (info1);
-		g_object_unref (info2);
-		return FALSE;
-	}
-
-	res = strcmp (id1, id2) == 0;
-	
-	g_object_unref (info1);
-	g_object_unref (info2);
-	
-	return res;
+	return ret;
 }
 
 void
@@ -365,10 +435,11 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 {
 	gboolean same_fs;
 	gboolean target_is_source_parent;
+	gboolean source_deletable;
 	const char *dropped_uri;
 	GFile *target, *dropped;
 	GdkDragAction actions;
-	NautilusFile *target_file;
+	NautilusFile *dropped_file, *target_file;
 
 	if (target_uri_string == NULL) {
 		*action = 0;
@@ -389,7 +460,8 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 	}
 	
 	dropped_uri = ((NautilusDragSelectionItem *)items->data)->uri;
-	target_file = nautilus_file_get_existing_by_uri (dropped_uri);
+	dropped_file = nautilus_file_get_existing_by_uri (dropped_uri);
+	target_file = nautilus_file_get_existing_by_uri (target_uri_string);
 	
 	/*
 	 * Check for trash URI.  We do a find_directory for any Trash directory.
@@ -402,38 +474,55 @@ nautilus_drag_default_drop_action_for_icons (GdkDragContext *context,
 			*action = GDK_ACTION_MOVE;
 		}
 
+		nautilus_file_unref (dropped_file);
 		nautilus_file_unref (target_file);
 		return;
 
-	} else if (target_file != NULL && nautilus_file_is_launcher (target_file)) {
+	} else if (dropped_file != NULL && nautilus_file_is_launcher (dropped_file)) {
 		if (actions & GDK_ACTION_MOVE) {
 			*action = GDK_ACTION_MOVE;
 		}
+		nautilus_file_unref (dropped_file);
 		nautilus_file_unref (target_file);
 		return;
 	} else if (eel_uri_is_desktop (target_uri_string)) {
 		target = nautilus_get_desktop_location ();
+
+		nautilus_file_unref (target_file);
+		target_file = nautilus_file_get (target);
+
 		if (eel_uri_is_desktop (dropped_uri)) {
 			/* Only move to Desktop icons */
 			if (actions & GDK_ACTION_MOVE) {
 				*action = GDK_ACTION_MOVE;
 			}
 			
+			g_object_unref (target);
+			nautilus_file_unref (dropped_file);
 			nautilus_file_unref (target_file);
 			return;
 		}
+	} else if (target_file != NULL && nautilus_file_is_archive (target_file)) {
+		*action = GDK_ACTION_COPY;
+
+		nautilus_file_unref (dropped_file);
+		nautilus_file_unref (target_file);
+		return;
 	} else {
 		target = g_file_new_for_uri (target_uri_string);
 	}
 
+	same_fs = check_same_fs (target_file, dropped_file);
+
+	nautilus_file_unref (dropped_file);
 	nautilus_file_unref (target_file);
 	
 	/* Compare the first dropped uri with the target uri for same fs match. */
 	dropped = g_file_new_for_uri (dropped_uri);
-	same_fs = check_same_fs (target, dropped);
 	target_is_source_parent = g_file_has_prefix (dropped, target);
-	
-	if (same_fs || target_is_source_parent ||
+	source_deletable = source_is_deletable (dropped);
+
+	if ((same_fs && source_deletable) || target_is_source_parent ||
 	    g_file_has_uri_scheme (dropped, "trash")) {
 		if (actions & GDK_ACTION_MOVE) {
 			*action = GDK_ACTION_MOVE;
@@ -598,6 +687,7 @@ nautilus_drag_drag_data_get (GtkWidget *widget,
 	gtk_selection_data_set (selection_data,
 				selection_data->target,
 				8, result->str, result->len);
+	g_string_free (result, TRUE);
 
 	return TRUE;
 }
@@ -908,3 +998,297 @@ nautilus_drag_selection_includes_special_link (GList *selection_list)
 	
 	return FALSE;
 }
+
+static void
+slot_proxy_drag_motion (GtkWidget          *widget,
+			GdkDragContext     *context,
+			int                 x,
+			int                 y,
+			unsigned int        time,
+			gpointer            user_data)
+{
+	NautilusDragSlotProxyInfo *drag_info;
+	NautilusWindowSlotInfo *target_slot;
+	GtkWidget *window;
+	GdkAtom target;
+	int action;
+	char *target_uri;
+
+	drag_info = user_data;
+
+	action = 0;
+
+	if (gtk_drag_get_source_widget (context) == widget) {
+		goto out;
+	}
+
+	window = gtk_widget_get_toplevel (widget);
+	g_assert (NAUTILUS_IS_WINDOW_INFO (window));
+
+	if (!drag_info->have_data) {
+		target = gtk_drag_dest_find_target (widget, context, NULL);
+		gtk_drag_get_data (widget, context, target, time);
+	}
+
+	target_uri = NULL;
+	if (drag_info->target_location != NULL) {
+		target_uri = g_file_get_uri (drag_info->target_location);
+	} else {
+		if (drag_info->target_slot != NULL) {
+			target_slot = drag_info->target_slot;
+		} else {
+			target_slot = nautilus_window_info_get_active_slot (NAUTILUS_WINDOW_INFO (window));
+		}
+
+		if (target_slot != NULL) {
+			target_uri = nautilus_window_slot_info_get_current_location (target_slot);
+		}
+	}
+
+	if (drag_info->have_data &&
+	    drag_info->have_valid_data) {
+		if (drag_info->info == NAUTILUS_ICON_DND_GNOME_ICON_LIST) {
+			nautilus_drag_default_drop_action_for_icons (context, target_uri,
+								     drag_info->data.selection_list,
+								     &action);
+		} else if (drag_info->info == NAUTILUS_ICON_DND_URI_LIST) {
+			action = nautilus_drag_default_drop_action_for_uri_list (context, target_uri);
+		} else if (drag_info->info == NAUTILUS_ICON_DND_NETSCAPE_URL) {
+			action = nautilus_drag_default_drop_action_for_netscape_url (context);
+		}
+	}
+
+	g_free (target_uri);
+
+out:
+	if (action != 0) {
+		gtk_drag_highlight (widget);
+	} else {
+		gtk_drag_unhighlight (widget);
+	}
+
+	gdk_drag_status (context, action, time);
+}
+
+static void
+drag_info_clear (NautilusDragSlotProxyInfo *drag_info)
+{
+	if (!drag_info->have_data) {
+		goto out;
+	}
+
+	if (drag_info->info == NAUTILUS_ICON_DND_GNOME_ICON_LIST) {
+		nautilus_drag_destroy_selection_list (drag_info->data.selection_list);
+	} else if (drag_info->info == NAUTILUS_ICON_DND_URI_LIST) {
+		g_list_free (drag_info->data.uri_list);
+	} else if (drag_info->info == NAUTILUS_ICON_DND_NETSCAPE_URL) {
+		g_free (drag_info->data.netscape_url);
+	}
+
+out:
+	drag_info->have_data = FALSE;
+	drag_info->have_valid_data = FALSE;
+
+	drag_info->drop_occured = FALSE;
+}
+
+static void
+slot_proxy_drag_leave (GtkWidget          *widget,
+		       GdkDragContext     *context,
+		       unsigned int        time,
+		       gpointer            user_data)
+{
+	NautilusDragSlotProxyInfo *drag_info;
+
+	drag_info = user_data;
+
+	gtk_drag_unhighlight (widget);
+	drag_info_clear (drag_info);
+}
+
+static gboolean
+slot_proxy_drag_drop (GtkWidget          *widget,
+		      GdkDragContext     *context,
+		      int                 x,
+		      int                 y,
+		      unsigned int        time,
+		      gpointer            user_data)
+{
+	GdkAtom target;
+	NautilusDragSlotProxyInfo *drag_info;
+
+	drag_info = user_data;
+	g_assert (!drag_info->have_data);
+
+	drag_info->drop_occured = TRUE;
+
+	target = gtk_drag_dest_find_target (widget, context, NULL);
+	gtk_drag_get_data (widget, context, target, time);
+
+	return TRUE;
+}
+
+
+static void
+slot_proxy_handle_drop (GtkWidget                *widget,
+			GdkDragContext           *context,
+			unsigned int              time,
+			NautilusDragSlotProxyInfo *drag_info)
+{
+	GtkWidget *window;
+	NautilusWindowSlotInfo *target_slot;
+	NautilusView *target_view;
+	char *target_uri;
+	GList *uri_list;
+
+	if (!drag_info->have_data ||
+	    !drag_info->have_valid_data) {
+		gtk_drag_finish (context, FALSE, FALSE, time);
+		drag_info_clear (drag_info);
+		return;
+	}
+
+	window = gtk_widget_get_toplevel (widget);
+	g_assert (NAUTILUS_IS_WINDOW_INFO (window));
+
+	if (drag_info->target_slot != NULL) {
+		target_slot = drag_info->target_slot;
+	} else {
+		target_slot = nautilus_window_info_get_active_slot (NAUTILUS_WINDOW_INFO (window));
+	}
+
+	target_uri = NULL;
+	if (drag_info->target_location != NULL) {
+		target_uri = g_file_get_uri (drag_info->target_location);
+	} else if (target_slot != NULL) {
+		target_uri = nautilus_window_slot_info_get_current_location (target_slot);
+	}
+
+	target_view = NULL;
+	if (target_slot != NULL) {
+		target_view = nautilus_window_slot_info_get_current_view (target_slot);
+	}
+
+	if (target_slot != NULL && target_view != NULL) {
+		if (drag_info->info == NAUTILUS_ICON_DND_GNOME_ICON_LIST) {
+			uri_list = nautilus_drag_uri_list_from_selection_list (drag_info->data.selection_list);
+			g_assert (uri_list != NULL);
+
+			nautilus_view_drop_proxy_received_uris (target_view,
+								uri_list,
+								target_uri,
+								context->action);
+			eel_g_list_free_deep (uri_list);
+		} else if (drag_info->info == NAUTILUS_ICON_DND_URI_LIST) {
+			nautilus_view_drop_proxy_received_uris (target_view,
+								drag_info->data.uri_list,
+								target_uri,
+								context->action);
+		} if (drag_info->info == NAUTILUS_ICON_DND_NETSCAPE_URL) {
+			nautilus_view_drop_proxy_received_netscape_url (target_view,
+									drag_info->data.netscape_url,
+									target_uri,
+									context->action);
+		}
+
+
+		gtk_drag_finish (context, TRUE, FALSE, time);
+	} else {
+		gtk_drag_finish (context, FALSE, FALSE, time);
+	}
+
+	if (target_view != NULL) {
+		g_object_unref (target_view);
+	}
+
+	g_free (target_uri);
+
+	drag_info_clear (drag_info);
+}
+
+static void
+slot_proxy_drag_data_received (GtkWidget          *widget,
+			       GdkDragContext     *context,
+			       int                 x,
+			       int                 y,
+			       GtkSelectionData   *data,
+			       unsigned int        info,
+			       unsigned int        time,
+			       gpointer            user_data)
+{
+	NautilusDragSlotProxyInfo *drag_info;
+	char **uris;
+
+	drag_info = user_data;
+
+	g_assert (!drag_info->have_data);
+
+	drag_info->have_data = TRUE;
+	drag_info->info = info;
+
+	if (data->length < 0) {
+		drag_info->have_valid_data = FALSE;
+		return;
+	}
+
+	if (info == NAUTILUS_ICON_DND_GNOME_ICON_LIST) {
+		drag_info->data.selection_list = nautilus_drag_build_selection_list (data);
+
+		drag_info->have_valid_data = drag_info->data.selection_list != NULL;
+	} else if (info == NAUTILUS_ICON_DND_URI_LIST) {
+		uris = gtk_selection_data_get_uris (data);
+		drag_info->data.uri_list = nautilus_drag_uri_list_from_array ((const char **) uris);
+		g_strfreev (uris);
+
+		drag_info->have_valid_data = drag_info->data.uri_list != NULL;
+	} else if (info == NAUTILUS_ICON_DND_NETSCAPE_URL) {
+		drag_info->data.netscape_url = g_strdup ((char *) data->data);
+
+		drag_info->have_valid_data = drag_info->data.netscape_url != NULL;
+	}
+
+	if (drag_info->drop_occured) {
+		slot_proxy_handle_drop (widget, context, time, drag_info);
+	}
+}
+
+void
+nautilus_drag_slot_proxy_init (GtkWidget *widget,
+			       NautilusDragSlotProxyInfo *drag_info)
+{
+        const GtkTargetEntry targets[] = {
+		{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST },
+		{ NAUTILUS_ICON_DND_NETSCAPE_URL_TYPE, 0, NAUTILUS_ICON_DND_NETSCAPE_URL }
+        };
+	GtkTargetList *target_list;
+
+	g_assert (GTK_IS_WIDGET (widget));
+	g_assert (drag_info != NULL);
+
+	gtk_drag_dest_set (widget, 0,
+			   NULL, 0,
+			   GDK_ACTION_MOVE |
+			   GDK_ACTION_COPY |
+			   GDK_ACTION_LINK |
+			   GDK_ACTION_ASK);
+
+	target_list = gtk_target_list_new (targets, G_N_ELEMENTS (targets));
+	gtk_target_list_add_uri_targets (target_list, NAUTILUS_ICON_DND_URI_LIST);
+	gtk_drag_dest_set_target_list (widget, target_list);
+	gtk_target_list_unref (target_list);
+
+	g_signal_connect (widget, "drag-motion",
+			  G_CALLBACK (slot_proxy_drag_motion),
+			  drag_info);
+	g_signal_connect (widget, "drag-drop",
+			  G_CALLBACK (slot_proxy_drag_drop),
+			  drag_info);
+	g_signal_connect (widget, "drag-data-received",
+			  G_CALLBACK (slot_proxy_drag_data_received),
+			  drag_info);
+	g_signal_connect (widget, "drag-leave",
+			  G_CALLBACK (slot_proxy_drag_leave),
+			  drag_info);
+}
+
+
