@@ -26,7 +26,6 @@
 #define NAUTILUS_ICON_CONTAINER_PRIVATE_H
 
 #include <eel/eel-glib-extensions.h>
-#include <libgnomeui/gnome-icon-item.h>
 #include <libnautilus-private/nautilus-icon-canvas-item.h>
 #include <libnautilus-private/nautilus-icon-container.h>
 #include <libnautilus-private/nautilus-icon-dnd.h>
@@ -214,10 +213,7 @@ struct NautilusIconContainerDetails {
 	GdkColor label_colors [LAST_LABEL_COLOR];
 	
 	/* State used so arrow keys don't wander if icons aren't lined up.
-	 * Keeps track of last axis arrow key was used on.
 	 */
-	Axis arrow_key_axis;
-	int arrow_key_start;
 	int arrow_key_start_x;
 	int arrow_key_start_y;
 	GtkDirectionType arrow_key_direction;
@@ -273,7 +269,11 @@ struct NautilusIconContainerDetails {
 	guint a11y_item_action_idle_handler;
 	GQueue* a11y_item_action_queue;
 
-	eel_boolean_bit is_reloading : 1;
+	eel_boolean_bit is_loading : 1;
+
+	eel_boolean_bit store_layout_timestamps : 1;
+	eel_boolean_bit store_layout_timestamps_when_finishing_new_icons : 1;
+	time_t layout_timestamp;
 
 	/* interactive search */
 	gboolean disable_popdown;
